package com.sf.vertx.handle;

import com.sf.vertx.api.pojo.ApiConfig;
import com.sf.vertx.enums.GatewayError;
import com.sf.vertx.exception.ServiceException;

import com.sf.vertx.api.pojo.AppConfig;

import com.sf.vertx.utils.AppUtils;
import io.vertx.ext.web.RoutingContext;
import lombok.extern.slf4j.Slf4j;

import static com.sf.vertx.constans.SACConstants.*;

@Slf4j
public class OpenParameterCheckHandlerImpl implements ParameterCheckHandler {

	@Override
	public void handle(RoutingContext rc) {
		try {
			log.info("Enter OPEN Route");
			// 判断OPEN模式, header不传递参数, 走域名映射
			String domain = rc.request().authority().host();
			log.info("request domain:{}", domain);
			AppConfig appConfig = AppConfigHandler.getAppConfigByDomain(domain);
			if (appConfig == null) {
				rc.fail(new ServiceException(GatewayError.APP_SERVICE_NOT_FOUND));
				return;
			}
			String apiCacheKey = appConfig.getAppCode() + CACHE_KEY_CONNECTOR + domain;
			ApiConfig apiConfig = AppConfigHandler.getApicodeConfig(apiCacheKey);
			if (apiConfig == null) {
				rc.fail(new ServiceException(GatewayError.API_SERVICE_NOT_FOUND));
				return;
			}
			// 设置应用配置、接口配置到上下文
			AppUtils.setAppConfigToRoutingContext(appConfig,rc);
			AppUtils.setApiConfigIntoRoutingContext(apiConfig,rc);
			rc.put(API_SERVICE_TYPE,AppConfigHandler.getServiceTypeOpen(apiCacheKey));
			// 将appcode和apicode设置到请求头，兼容原有逻辑
			rc.request().headers().add(AppConfigHandler.getAppCodeHeaderKey(), appConfig.getAppCode());
			rc.request().headers().add(AppConfigHandler.getApiCodeHeaderKey(), apiConfig.getApiCode());
		} catch (Exception e) {
			log.error("OpenParameterCheckHandlerImpl Error:",e);
			rc.fail(new ServiceException(GatewayError.DEFAULT_SERVICE_ERROR));
			return;
		}
		rc.next();
	}
}
