package com.sf.vertx.enums;

/**
 * VertX网关错误
 *
 * @author zoukun
 */
public enum GatewayError {
    // CLIENT ERROR
    BAD_REQUEST(400, "Bad Request", ErrorType.CLIENT_ERROR),
    UNAUTHORIZED(401, "Unauthorized", ErrorType.CLIENT_ERROR),
    FORBIDDEN(403, "Forbidden", ErrorType.CLIENT_ERROR),
    NOT_FOUND(404, "Not Found", ErrorType.CLIENT_ERROR),
    METHOD_NOT_ALLOWED(405, "Method Not Allowed", ErrorType.CLIENT_ERROR),
    NOT_ACCEPTABLE(406, "Not Acceptable", ErrorType.CLIENT_ERROR),
    PROXY_AUTHENTICATION_REQUIRED(407, "Proxy Authentication Required", ErrorType.CLIENT_ERROR),
    REQUEST_TIMEOUT(408, "Request Timeout", ErrorType.CLIENT_ERROR),
    CONFLICT(409, "Conflict", ErrorType.CLIENT_ERROR),
    GONE(410, "Gone", ErrorType.CLIENT_ERROR),
    LENGTH_REQUIRED(411, "Length Required", ErrorType.CLIENT_ERROR),
    PRECONDITION_FAILED(412, "Precondition Failed", ErrorType.CLIENT_ERROR),
    PAYLOAD_TOO_LARGE(413, "Payload Too Large", ErrorType.CLIENT_ERROR),
    URI_TOO_LONG(414, "URI Too Long", ErrorType.CLIENT_ERROR),
    UNSUPPORTED_MEDIA_TYPE(415, "Unsupported Media Type", ErrorType.CLIENT_ERROR),
    REQUESTED_RANGE_NOT_SATISFIABLE(416, "Requested range not satisfiable", ErrorType.CLIENT_ERROR),
    EXPECTATION_FAILED(417, "Expectation Failed", ErrorType.CLIENT_ERROR),
    UNPROCESSABLE_ENTITY(422, "Unprocessable Entity", ErrorType.CLIENT_ERROR),
    LOCKED(423, "Locked", ErrorType.CLIENT_ERROR),
    FAILED_DEPENDENCY(424, "Failed Dependency", ErrorType.CLIENT_ERROR),
    TOO_EARLY(425, "Too Early", ErrorType.CLIENT_ERROR),
    UPGRADE_REQUIRED(426, "Upgrade Required", ErrorType.CLIENT_ERROR),
    PRECONDITION_REQUIRED(428, "Precondition Required", ErrorType.CLIENT_ERROR),
    TOO_MANY_REQUESTS(429, "Too Many Requests", ErrorType.CLIENT_ERROR),
    REQUEST_HEADER_FIELDS_TOO_LARGE(431, "Request Header Fields Too Large", ErrorType.CLIENT_ERROR),
    UNAVAILABLE_FOR_LEGAL_REASONS(451, "Unavailable For Legal Reasons", ErrorType.CLIENT_ERROR),

    // SERVER_ERROR
    INTERNAL_SERVER_ERROR(500, "Internal Server Error", ErrorType.SERVER_ERROR),
    NOT_IMPLEMENTED(501, "Not Implemented", ErrorType.SERVER_ERROR),
    BAD_GATEWAY(502, "Bad Gateway", ErrorType.SERVER_ERROR),
    SERVICE_UNAVAILABLE(503, "Service Unavailable", ErrorType.SERVER_ERROR),
    GATEWAY_TIMEOUT(504, "Gateway Timeout", ErrorType.SERVER_ERROR),
    HTTP_VERSION_NOT_SUPPORTED(505, "HTTP Version not supported", ErrorType.SERVER_ERROR),
    INSUFFICIENT_STORAGE(507, "Insufficient Storage", ErrorType.SERVER_ERROR),
    NOT_EXTENDED(510, "Not Extended", ErrorType.SERVER_ERROR),
    NETWORK_AUTHENTICATION_REQUIRED(511, "Network Authentication Required", ErrorType.SERVER_ERROR),

    // SERVICE_ERROR
    DEFAULT_SERVICE_ERROR(10000, "Service request failed, please try again later", ErrorType.SERVICE_ERROR), // 默认业务错误提示
    APP_ACCESS_PROHIBITED(10001, "应用禁止访问,请联系管理员", ErrorType.SERVICE_ERROR),
    APP_SERVICE_NOT_FOUND(10002, "APP service not found", ErrorType.SERVICE_ERROR),
    API_SERVICE_NOT_FOUND(10003, "API service not found", ErrorType.SERVICE_ERROR),
    PARAMETER_TRANSFER_ERROR(10010, "参数传递错误", ErrorType.SERVICE_ERROR),
    UNABLE_TO_FIND_ROUTING_ADDRESS(10011, "无法找到路由地址", ErrorType.SERVICE_ERROR),
    UNABLE_TO_FIND_MATCHING_ENCRYPTION_ALGORITHM(10012, "无法找到匹配的加解密算法", ErrorType.SERVICE_ERROR),
    UNABLE_TO_FIND_MATCHING_CIRCUIT_BREAKER_STRATEGY(10013, "无法找到匹配的熔断策略", ErrorType.SERVICE_ERROR),
    REVERSE_PROXY_EXECUTION_ERROR(10014, "反向代理执行错误", ErrorType.SERVICE_ERROR),
    REQUEST_URL_RESTRICTED_BY_FLOW(10015, "请求url被限流", ErrorType.SERVICE_ERROR),
    REQUEST_URL_IS_BROKEN(10016, "请求url被熔断", ErrorType.SERVICE_ERROR),
    APP_REQUEST_URL_RESTRICTED_BY_FLOW(10017, "应用请求url被限流", ErrorType.SERVICE_ERROR),
    ;

    private final int code;
    private final String reasonPhrase;

    private final ErrorType errorType;

    GatewayError(int code, String reasonPhrase, ErrorType errorType) {
        this.code = code;
        this.reasonPhrase = reasonPhrase;
        this.errorType = errorType;
    }

    public int getCode() {
        return code;
    }

    public ErrorType getErrorType() {
        return errorType;
    }

    public String getReasonPhrase() {
        return reasonPhrase;
    }

    public static GatewayError getByCode(int code) {
        for (GatewayError value : GatewayError.values()) {
            if (value.code == code) {
                return value;
            }
        }
        return null;
    }

    /**
     * 网关错误类型
     */
    public enum ErrorType {

        CLIENT_ERROR(4),
        SERVER_ERROR(5),
        /**
         * 业务错误，建议业务错误返回HTTP状态码为200，业务码放在返回信息当中。
         */
        SERVICE_ERROR(10),
        ;

        private final int value;

        ErrorType(int value) {
            this.value = value;
        }

        public int value() {
            return this.value;
        }

    }
}
